/************************************************************
* $Id: nfiles.c,v 1.4 2005/10/11 13:49:25 mechanoid Exp $
*
*   neural net based associative memory
*       
*       OS : FreeBSD 4.6
* COMPILER : GNU project C and C++ Compiler (gcc-2.95.3)
*
*   AUTHOR : Evgeny S. Borisov
*
*  Glushkov Institute of Cybernetics
*  National Academy of Sciences of Ukraine
* 
*    http://mechanoid.narod.ru
*  e-mail : periodical@ukrpost.net
* 
************************************************************/
#include <gdk/gdk.h>
#include <errno.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/uio.h>
#include <unistd.h>
#include <fcntl.h>

#include "nparam.h"
#include "nnet.h"
#include "nfiles.h"

GdkPixbuf *pixbuf=NULL;

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* load image from file to buffer                                  */
GdkPixbuf* img_load(char filename[]) {
   GdkPixbuf *pb;
   GError  *err = NULL;

   pb = gdk_pixbuf_new_from_file(filename,&err);
   if(err!=NULL) { fprintf (stderr, err->message); g_error_free(err); return NULL; }
   return pb;
}
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* save image from buffer to file (BMP)                            */
int img_save(GdkPixbuf *pb,char filename[]) {
   GError *err = NULL;

   g_return_val_if_fail(pb != NULL, 1);
   gdk_pixbuf_save(pb,filename, "bmp", &err,NULL);
   if(err!=NULL) { fprintf (stderr, err->message); g_error_free(err); return 2; }
   
   return 0;
}
/* - - - - - - - - - - - - - - - - - - - */
/* load nnet input from image file       */
int nrdata_load(char filename[]) {
   guchar *buffer;
   int i,j,k;
   
   if(pixbuf != NULL){gdk_pixbuf_unref(pixbuf); pixbuf = NULL;}
   pixbuf=img_load(filename);
   g_return_val_if_fail(pixbuf != NULL, 1);
   buffer=gdk_pixbuf_get_pixels(pixbuf);
   g_return_val_if_fail(buffer != NULL, 2);
   
   for( k=0,j=0 ; j<IMG_HEIGHT ; j++ ){
	  for( i=0 ; i<IMG_ROWS ; i++ ) { 
		 output[k++]=buffer[i+(IMG_ROWS+IMG_NROWS)*j]>0?LO:HI;
	  }
   }
   return 0; 
}
/* - - - - - - - - - - - - - - - - - - - */
/* save nnet output to image file       */
int nrdata_save(char filename[]){
   guchar *buffer;
   int i,j,k;
   
   g_return_val_if_fail(pixbuf != NULL, 1);
   buffer=gdk_pixbuf_get_pixels(pixbuf);
   g_return_val_if_fail(buffer != NULL, 2);
   
   for( k=0,j=0 ; j<IMG_HEIGHT ; j++ ){
	  for( i=0 ; i<IMG_ROWS ; i++ ) { 
		 buffer[i+(IMG_ROWS+IMG_NROWS)*j]=output[k++];
	  }
   }
   return img_save(pixbuf,filename);
}
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* save weight matrix to file                                      */
int weight_save(char filename[]){
   ssize_t sz;
   int file;

   file=open(filename,O_WRONLY|O_TRUNC|O_CREAT);
   if(file<0){ fprintf(stderr,"%s : %s\n",filename,strerror(errno)); return 1; }
   sz=write(file, weight, OUTSIZE*OUTSIZE*sizeof(weight_t));
   if(sz!= OUTSIZE*OUTSIZE*sizeof(weight_t)){
  	  fprintf(stderr,"%s : %s\n",filename,strerror(errno));
	  close(file);
	  return 2;
   }
   close(file);

   if(chmod(filename,S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH)) {
	  fprintf(stderr,"%s : %s\n",filename,strerror(errno));
   }
   
   return 0;
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* load weight matrix from file                                      */
int weight_load(char filename[]){
   ssize_t sz;
   int file;

   file=open(filename,O_RDONLY);
   if(file<0){fprintf(stderr,"%s : %s\n",filename,strerror(errno));return 1;}
   sz=read(file, weight, OUTSIZE*OUTSIZE*sizeof(weight_t));
   if(sz!= OUTSIZE*OUTSIZE*sizeof(weight_t)){
  	  fprintf(stderr,"%s : %s\n",filename,strerror(errno));
	  close(file);
	  return 2;
   }
   close(file);
   return 0;
}
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* print weight matrix                                             */
void weight_print(){
   int i,j;  
   for(i=0 ; i<OUTSIZE ; i++) {
	  for(j=0 ; j<OUTSIZE ; j++) { fprintf(stdout,"%i ",weight[i][j]); }
	  fprintf(stdout,"\n");
   }
}
/* - - - - - - - - - - - - - - - - - - - */
/* print nnet output                     */
void nrdata_print(){
   int i;  
   for( i=0 ; i<OUTSIZE ; i++ ) { fprintf(stdout,"%i ",output[i]); }
   fprintf(stdout,"\n");
}

/* - - - - - - - - - - - - - - - - - - - */
int is_bmp(char filename[]) {
   char *file_ext;
   int len;

   if((len=strlen(filename))<5) {return 0;}
   file_ext=filename+len-4;
   if(!memcmp(file_ext,".bmp",4)){ return 1; }
   
   return 0;
}
/* - - - - - - - - - - - - - - - - - - - */
int is_folder(char filename[]) {
   struct stat *sb;
   int res=0;
  
   sb=malloc(sizeof(sb));
   if(sb==NULL) {
	  fprintf(stderr,"malloc : %s\n",strerror(errno));
	  return 0;
   }
   if(stat(filename, sb)){
	    fprintf(stderr,"%s : %s\n",filename,strerror(errno));
   } else {
	   res=sb->st_mode & S_IFDIR;
   }
   free(sb);
   return res;
}
