/************************************************************
* $Id: nnet.c,v 1.4 2005/10/11 20:46:18 mechanoid Exp $
*       
*   neural net based associative memory
*
*       OS : FreeBSD 4.6
* COMPILER : GNU project C and C++ Compiler (gcc-2.95.3)
*
*   AUTHOR : Evgeny S. Borisov
*
*  Glushkov Institute of Cybernetics
*  National Academy of Sciences of Ukraine
* 
*    http://mechanoid.narod.ru
*  e-mail : periodical@ukrpost.net
* 
************************************************************/
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "nnet.h"

data_t   output_prev[OUTSIZE];  // prev. output vector
data_t   output[OUTSIZE]; // output vector
weight_t weight[OUTSIZE][OUTSIZE]; // weight matrix

/* - - - - - - - - - - - - - - - - - - - */
/* activation function                   */
data_t activate(weight_t s) { 
   return ((s>T)?HI:LO);
}

/* - - - - - - - - - - - - - - - - - - - */
/* check stability net output            */
/* return 0 if net output is stability   */
int stability() { 
   return (memcmp(output,output_prev,OUTSIZE)); 
}

/* - - - - - - - - - - - - - - - - - - - */
/*  init weight matrix                   */
void weight_init() { 
   memset(weight,0,OUTSIZE*OUTSIZE); 
}

/* - - - - - - - - - - - - - - - - - - - */
/* calculate state of neuron number <out>  */
weight_t state(int out) {
   weight_t s=0.0;
   int i=0;
   
   for(i=0;i<OUTSIZE;i++){ 
	  s+=(weight_t)output_prev[i]*weight[out][i]; 
   }

   return s;
}

/* - - - - - - - - - - - - - - - - - - - */
/* calculate net output                  */
void step() {
   int i;
   memcpy(output_prev,output,OUTSIZE);	
   for(i=0;i<OUTSIZE;i++) { output[i]=activate(state(i)); }
}

/* - - - - - - - - - - - - - - - - - - - */
/* remember input                        */
void store() {
   int i,j;
   for(j=0 ; j<OUTSIZE ; j++) {
	  for(i=0 ; i<OUTSIZE ; i++) {
		 if(i==j) {
			weight[i][i]=0;
		 } else {
			weight[j][i]+=output[i]*output[j];
		 }
	  }
   }
}

/* - - - - - - - - - - - - - - - - - - - */
/* recall stored pattern                 */
void restore() {
   int i;
   for( i=0 ; 
		(i<MAX_ITERATION) && stability() ;
		i++,step()
		);
}
